package solutions.bitbadger.documents.groovy.jvm

import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
//import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.Field
import solutions.bitbadger.documents.Parameter
import solutions.bitbadger.documents.ParameterType
import solutions.bitbadger.documents.jvm.Parameters
import solutions.bitbadger.documents.support.ForceDialect

import static groovy.test.GroovyAssert.*

/**
 * Unit tests for the `Parameters` object
 */
@DisplayName('JVM | Groovy | Parameters')
class ParametersTest {

    /**
     * Reset the dialect
     */
    @AfterEach
    void cleanUp() {
        ForceDialect.none()
    }

    @Test
    @DisplayName('nameFields works with no changes')
    void nameFieldsNoChange() {
        def fields = List.of(Field.equal('a', '', ':test'), Field.exists('q'), Field.equal('b', '', ':me'))
        def named = Parameters.nameFields(fields).toList()
        assertEquals('There should have been 3 fields in the list', fields.size(), named.size())
        assertSame('The first field should be the same', fields[0], named[0])
        assertSame('The second field should be the same', fields[1], named[1])
        assertSame('The third field should be the same', fields[2], named[2])
    }

    @Test
    @DisplayName('nameFields works when changing fields')
    void nameFieldsChange() {
        def fields = List.of(Field.equal('a', ''), Field.equal('e', '', ':hi'), Field.equal('b', ''),
                Field.notExists('z'))
        def named = Parameters.nameFields(fields).toList()
        assertEquals('There should have been 4 fields in the list', fields.size(), named.size())
        assertNotSame('The first field should not be the same', fields[0], named[0])
        assertEquals('First parameter name incorrect', ':field0', named[0].parameterName)
        assertSame('The second field should be the same', fields[1], named[1])
        assertNotSame('The third field should not be the same', fields[2], named[2])
        assertEquals('Third parameter name incorrect', ':field1', named[2].parameterName)
        assertSame('The fourth field should be the same', fields[3], named[3])
    }

    @Test
    @DisplayName('replaceNamesInQuery replaces successfully')
    void replaceNamesInQuery() {
        def parameters = List.of(new Parameter(':data', ParameterType.JSON, '{}'),
                new Parameter(':data_ext', ParameterType.STRING, ''))
        def query = 'SELECT data, data_ext FROM tbl WHERE data = :data AND data_ext = :data_ext AND more_data = :data'
        assertEquals('Parameters not replaced correctly',
                'SELECT data, data_ext FROM tbl WHERE data = ? AND data_ext = ? AND more_data = ?',
                Parameters.replaceNamesInQuery(query, parameters))
    }

    @Test
    @DisplayName('fieldNames generates a single parameter (PostgreSQL)')
    void fieldNamesSinglePostgres() {
        ForceDialect.postgres()
        def nameParams = Parameters.fieldNames(List.of('test')).toList()
        assertEquals('There should be one name parameter', 1, nameParams.size())
        assertEquals('The parameter name is incorrect', ':name', nameParams[0].name)
        assertEquals('The parameter type is incorrect', ParameterType.STRING, nameParams[0].type)
        assertEquals('The parameter value is incorrect', '{test}', nameParams[0].value)
    }

    @Test
    @DisplayName('fieldNames generates multiple parameters (PostgreSQL)')
    void fieldNamesMultiplePostgres() {
        ForceDialect.postgres()
        def nameParams = Parameters.fieldNames(List.of('test', 'this', 'today')).toList()
        assertEquals('There should be one name parameter', 1, nameParams.size())
        assertEquals('The parameter name is incorrect', ':name', nameParams[0].name)
        assertEquals('The parameter type is incorrect', ParameterType.STRING, nameParams[0].type)
        assertEquals('The parameter value is incorrect', '{test,this,today}', nameParams[0].value)
    }

    @Test
    @DisplayName('fieldNames generates a single parameter (SQLite)')
    void fieldNamesSingleSQLite() {
        ForceDialect.sqlite()
        def nameParams = Parameters.fieldNames(List.of('test')).toList()
        assertEquals('There should be one name parameter', 1, nameParams.size())
        assertEquals('The parameter name is incorrect', ':name0', nameParams[0].name)
        assertEquals('The parameter type is incorrect', ParameterType.STRING, nameParams[0].type)
        assertEquals('The parameter value is incorrect', 'test', nameParams[0].value)
    }

    @Test
    @DisplayName('fieldNames generates multiple parameters (SQLite)')
    void fieldNamesMultipleSQLite() {
        ForceDialect.sqlite()
        def nameParams = Parameters.fieldNames(List.of('test', 'this', 'today')).toList()
        assertEquals('There should be one name parameter', 3, nameParams.size())
        assertEquals('The first parameter name is incorrect', ':name0', nameParams[0].name)
        assertEquals('The first parameter type is incorrect', ParameterType.STRING, nameParams[0].type)
        assertEquals('The first parameter value is incorrect', 'test', nameParams[0].value)
        assertEquals('The second parameter name is incorrect', ':name1', nameParams[1].name)
        assertEquals('The second parameter type is incorrect', ParameterType.STRING, nameParams[1].type)
        assertEquals('The second parameter value is incorrect', 'this', nameParams[1].value)
        assertEquals('The third parameter name is incorrect', ':name2', nameParams[2].name)
        assertEquals('The third parameter type is incorrect', ParameterType.STRING, nameParams[2].type)
        assertEquals('The third parameter value is incorrect', 'today', nameParams[2].value)
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('fieldNames fails if dialect not set')
//    void fieldNamesFails() {
//        assertThrows(DocumentException) { Parameters.fieldNames(List.of()) }
//    }
}
